#ifndef __KYSEC_DLP_H__
#define __KYSEC_DLP_H__

#include <bits/types.h>
#include <linux/limits.h>

#ifdef __cplusplus
extern "C" {
#endif

// 序列化所需的最大内存空间
#define MAX_APP_RULE_SIZE   (PATH_MAX + sizeof(unsigned int))
#define MAX_RULE_SIZE       (MAX_APP_RULE_SIZE + sizeof(int)*2)
#define MAX_FILE_RULE_SIZE  (PATH_MAX + MAX_RULE_SIZE)

/* 描述
uid:        拥有权限的用户id
app_path：  存储应用程序的路径
acl：       应用程序对应的权限
*/  
typedef struct __app_rule__ {
        unsigned int uid;
        char *app_path;
        unsigned int acl;
} app_rule;

/*
描述 
app_rules:　规则列表
rule_size:　规则列表数量
*/
typedef struct __rule__ {
        app_rule *app_rules;
        int rule_size;
} rule;

/*
描述 
dlp_path: 保护路径
*/
typedef struct _dlp_rule_ {
        char *dlp_path;
        rule rule;
} dlp_rule;


#define FILE_READ (1 << 2)
#define FILE_WRITE (1 << 1)
#define FILE_DEL (1 << 0)


// 所有用户都可以访问Uid
#define USER_EVERYBODY ((__uid_t)(-3))

// 错误码
#define ERR_RULE_NOT_EXIST       -1
#define ERR_RULE_AND_DLP_PATH_NOT_EXIST -2
#define ERR_OPEN_PROCFS          -3
#define ERR_MAKEDIR              -4
#define ERR_PROCFS_IOCTL         -5
#define ERR_GET_CURRENT_APP_PATH -6
#define ERR_PATH                 -7
#define ERR_SET_RULE_FILE_RULE   -8
#define ERR_SERIALIZE            -9
#define ERR_INTERN_ERROR         -10
#define ERR_PARAM                -11
#define ERR_PERMISSION_DENIED    -12
#define ERR_HASH_ERROR           -13


/* 设置应用保护路径的规则。如果保护路径不存在，则创建保护路径文件夹。
*  如果应用保护路径规则已经存在，则会用传入规则替换存在的规则。
*  调用前，可通过dlp_get_path()来判断应用保护路径的规则是否已经存在。

*  注：dlp_path总是获取成功。
*  
* 参数：
*  rule：要设置的规则
*  dlp_path: 传入一段长度为PATH_MAX的字符数组，用来接收应用的保护路径

* 返回值：
*  0：  设置规则成功
*  其他： 设置保护路径规则失败
*/
int dlp_create(const rule * rule, char *dlp_path);

/* 获取当前应用的安全路径

*  注：dlp_path总是获取成功。

* 参数：
*  dlp_path: 传入一段长度为PATH_MAX的字符数组，用来接收应用的保护路径

* 返回值：
*  0: 当前应用的保护路径规则已经存在
*  -1: 当前应用的保护路径存在，规则不存在
*  -2：当前应用的保护路径和规则都不存在
*  其他： 当前应用的保护路径规则获取失败
*/
int dlp_get_path(char *dlp_path);

/* 获取其他应用的安全路径

*  注：dlp_path总是获取成功。

* 参数：
*  dlp_path: 传入一段长度为PATH_MAX的字符数组，用来接收应用的保护路径

* 返回值：
*  0: 应用的保护路径规则已经存在
*  -1: 应用的保护路径存在，规则不存在
*  -2：应用的保护路径和规则都不存在
*  其他： 应用的保护路径规则获取失败
*/
int dlp_get_path_by_app(const char* app_path, char* dlp_path);

/* 设置文件、文件夹保护规则 relative_file为相对保护路径的文件路径
*  如果文件的保护规则已经存在，则会替换。
*  如果传入的relative_file为空字符串"", 表示设置根保护路径的权限

* 参数：
*  app_path: 应用的路径
*  rule: 设置的规则

* 返回值：
*  0:  设置成功
*  其他： 设置失败
*/
int dlp_set_file_rule(const char *relative_file, const rule * rule);

/* 查询文件保护规则 relative_file为相对保护路径的文件路径
*  如果文件的保护规则已经存在，则会替换。
*  查询成功，rule需由用户调用dlp_free_rule释放

* 返回值：
*  0: 查询成功
*  其他： 查询失败
*/
int dlp_get_file_rule(const char *relative_file, rule ** rule);

/* rule释放 
*  dlp_get_file_rule获取的rule由调用者管理，需调此函数释放
*/
void dlp_free_rule(rule * r);

/* 递归删除应用的保护路径规则
*  不会删除保护路径文件夹

* 返回值：
*  0: 删除成功
*  其他： 删除失败
*/
int dlp_remove();

/* 获取所有拥有app保护路径的用户的数量
*  注意：如果用户的保护路径被删除，那么dlp_get_users_size数量将会改变
* 参数：
*  app_path: 应用的路径
*  users_size: 获取到的用户数量

* 返回值：
*  0:  获取成功
*  其他： 获取失败
*/
int dlp_get_users_size(const char* app_path, int* users_size);

/* 获取应用的第user_index个用户的保护路径，user_index从0开始 
*  注意：如果用户的保护路径被删除，那么dlp_get_users_size数量将会改变
*  所以如果需要删除保护路径的话，需要从最大的user_index用户开始查询然后删除
* 
* 参数：
*  app_path: 应用的路径
*  user_index: 表示第user_index个用户
*  dlp_path： 传入一段长度为PATH_MAX的字符数组，用来接收应用的保护路径

* 返回值：
*  0:  获取成功
*  其他： 获取失败
*/
int dlp_get_user_path(const char* app_path, int user_index, char* dlp_path);

/* 创建应用的共享保护路径。
*  共享保护路径，所有用户的应用均可以设置规则，所有的用户应用都可以访问。
*  如果保护路径不存在，则创建保护路径文件夹。
*  如果应用保护路径规则已经存在，则会用传入规则替换存在的规则。

*  注意：规则中的用户需设置为USER_EVERYBODY，表示对所有用户授权。
*
* 参数：
*  rule：要设置的规则
*  dlp_path: 传入一段长度为PATH_MAX的字符数组，用来接收应用的保护路径

* 返回值：
*  0：  设置规则成功
*  其他： 设置保护路径规则失败
*/
int dlp_create_common_path(const rule* rule, char* dlp_path);

/* 获取应用的共享安全路径

*  注：dlp_path总是获取成功。

* 参数：
*  dlp_path: 传入一段长度为PATH_MAX的字符数组，用来接收应用的保护路径

* 返回值：
*  0: 应用的保护路径规则已经存在
*  -1: 应用的保护路径存在，规则不存在
*  -2：应用的保护路径和规则都不存在
*  其他： 应用的保护路径规则获取失败
*/
int dlp_get_app_common_path(const char* app_path, char* dlp_path);

/* 设置共享文件、文件夹保护规则 relative_file为相对保护路径的文件路径
*  如果文件的保护规则已经存在，则会替换。
*  如果传入的relative_file为空字符串"", 表示设置根保护路径的权限

* 参数：
*  app_path: 应用的路径
*  rule: 设置的规则

* 返回值：
*  0:  设置成功
*  其他： 设置失败
*/
int dlp_set_common_file_rule(const char *relative_file, const rule * rule);

/* 查询共享文件保护规则 relative_file为相对保护路径的文件路径
*  
*  查询成功，rule需由用户调用dlp_free_rule释放

* 返回值：
*  0: 查询成功
*  其他： 查询失败
*/
int dlp_get_common_file_rule(const char *relative_file, rule ** rule);

#ifdef __cplusplus
}
#endif

#endif